name: Display label on push 
description: Forcing Label Names to Display on Defect Push
author: TechMatrix
version: 1.0
includes: ^runs/view/|^suites/view
excludes:

js:
(function() {
  'use strict';

  /**
   * ★各 mapping 用の設定（id属性またはfor属性に設定された値に対応）
   * [指定の形式] "idまたはforの属性値" : "表示するラベル文字列"
   */
  var labelTextMapping = {
    // Redmine (<label id="...">)
    "labeldefect_tracker": "トラッカー",
    "labeldefect_category": "カテゴリ",
    "labeldefect_assigned_to": "担当者",
    "labeldefect_estimated_hours": "予定工数",
    "labeldefect_custom_field_string_1": "カスタムフィールド：テキスト",
    "labeldefect_custom_field_list_2": "カスタムフィールド：リスト",
    "labeldefect_custom_field_string_4": "カスタムフィールド：整数",
    "labeldefect_custom_field_date_5": "カスタムフィールド：日付",
    // Redmine (<label for="...">)
    "defect_subject": "題名",
    "defect_custom_field_text_6": "カスタムフィールド：長いテキスト",
    "defect_custom_field_listmulti_3": "カスタムフィールド：リスト（複数選択）",
    "defect_custom_field_bool_7": "カスタムフィールド：真偽値",
    "defect_custom_field_user_8": "カスタムフィールド：ユーザー",
    "defect_custom_field_keyvalue_9": "カスタムフィールド：キーバリュー",
    "defect_custom_field_keyvaluemulti_11": "カスタムフィールド：キーバリュー（複数選択）",
    // JIRA (<label for="...">)
    "defect_customfield_10435": "カスタム日付",
    "defect_customfield_10436": "カスタムユーザー",
    // Backlog (<label for="...">)
    "defect_issue_type": "種別",
    "defect_category": "カテゴリー",
    "defect_milestone": "マイルストーン",
    "defect_version": "発生バージョン"
  };

  // DEBUG_MODE: 1 ならログ出力、0 ならログ出力しない
  var DEBUG_MODE = 0;
  function debugLog() {
    if (DEBUG_MODE) {
      console.log.apply(console, arguments);
    }
  }

  /**
   * 指定されたキーに対応する<label>要素を取得する。
   * まず document.getElementById(key) で取得するが、
   * もし取得された要素が <label> でなければ、<label for="key"> で再検索する。
   *
   * @param {string} key 対象となるキー（例："defect_subject" や "labeldefect_tracker"）
   * @returns {HTMLElement|null} 該当する<label>要素（存在しない場合はnull）
   */
  function getLabelByKey(key) {
    var elem = document.getElementById(key);
    if (elem && elem.tagName !== 'LABEL') {
      // 取得された要素が <label> でない場合は、for 属性で検索する
      elem = document.querySelector('label[for="' + key + '"]');
    }
    return elem;
  }

  /**
   * 指定されたキーに対応する<label>要素内にテキストを追加する。
   *
   * @param {string} key 対象となるキー（id または for 属性の値そのままで指定）
   * @param {string} newText 追加するテキスト
   * @param {boolean} [force=false] true の場合は、既存のテキストノードの有無に関わらず、
   *        先頭に newText を追加する（ただし、先頭テキストがすでに newText で始まっていれば何もしない）
   */
  function updateLabelText(key, newText, force) {
    force = force || false;
    var label = getLabelByKey(key);
    if (!label) return;

    if (force) {
      // 先頭がテキストノードで、かつ既に newText で始まっていれば何もしない
      if (label.firstChild && label.firstChild.nodeType === Node.TEXT_NODE) {
        if (label.firstChild.textContent.indexOf(newText) === 0) {
          return;
        }
      }
      // 強制的に newText を先頭に挿入する
      label.insertBefore(document.createTextNode(newText), label.firstChild);
    } else {
      // すでに空でないテキストノードがあるかチェック（空白のみは除外）
      var hasText = false;
      for (var i = 0; i < label.childNodes.length; i++) {
        var node = label.childNodes[i];
        if (node.nodeType === Node.TEXT_NODE && node.textContent.trim() !== "") {
          hasText = true;
          break;
        }
      }
      // テキストノードが存在しない場合のみ、先頭に newText を追加する
      if (!hasText) {
        label.insertBefore(document.createTextNode(newText), label.firstChild);
      }
    }
  }

  /**
   * MutationObserver を用いて、対象の<label>要素の DOM 変化を監視し、
   * テキストノードが空になった場合に再度 updateLabelText を実行する。
   *
   * @param {string} key 対象のラベルの id または for 属性の値
   */
  function observeLabel(key) {
    var label = getLabelByKey(key);
    if (!label) return;
    // マッピングテキストは labelTextMapping から取得
    var mappingText = labelTextMapping[key];
    if (!mappingText) return;

    var observer = new MutationObserver(function(mutations) {
      var hasText = false;
      for (var i = 0; i < label.childNodes.length; i++) {
        var node = label.childNodes[i];
        if (node.nodeType === Node.TEXT_NODE && node.textContent.trim() !== "") {
          hasText = true;
          break;
        }
      }
      if (!hasText) {
        updateLabelText(key, mappingText, false);
      }
    });
    var config = { childList: true, subtree: true, characterData: true };
    observer.observe(label, config);
  }

  /**
   * 初回のラベル更新処理
   */
  function run() {
    debugLog('run() start');
    Object.keys(labelTextMapping).forEach(function(key) {
      var label = getLabelByKey(key);
      if (!label) return;
      // <label for="..."> なら初回のみ強制更新（force=true）、それ以外は既存テキストが無い場合のみ更新
      if (label.hasAttribute("for")) {
        debugLog('updateLabelText for key (force):', key);
        updateLabelText(key, labelTextMapping[key], true);
      } else {
        debugLog('updateLabelText for key:', key);
        updateLabelText(key, labelTextMapping[key], false);
      }
    });
    // すべての対象 <label> について、DOM 変化を監視する
    Object.keys(labelTextMapping).forEach(function(key) {
      observeLabel(key);
    });
    debugLog('run() end');
  }

  /**
   * モーダル内の各<label>について、更新が完了するまでポーリングする。
   * ※一定回数（maxRetries）を超えたらタイマーを停止することで無限ループを防ぐ
   */
  function pollAndUpdateLabels() {
    var maxRetries = 10;
    var retries = 0;
    debugLog('pollAndUpdateLabels: Starting polling for label updates.');
    var interval = setInterval(function() {
      var allUpdated = true;
      var keys = Object.keys(labelTextMapping);
      keys.forEach(function(key) {
        var label = getLabelByKey(key);
        if (label) {
          var busy = label.querySelector('.busy');
          var text = "";
          if (busy && label.firstChild && label.firstChild.nodeType === Node.TEXT_NODE) {
            text = label.firstChild.textContent.trim();
          } else {
            text = label.textContent.trim();
          }
          debugLog('pollAndUpdateLabels: Label key="' + key + '" - current text:', text);
          if (text === "") {
            debugLog('pollAndUpdateLabels: Label key="' + key + '" is empty. Attempting update.');
            // for属性を持っている場合は force を true にする
            updateLabelText(key, labelTextMapping[key], label.hasAttribute("for"));
            if (busy && label.firstChild && label.firstChild.nodeType === Node.TEXT_NODE) {
              text = label.firstChild.textContent.trim();
            } else {
              text = label.textContent.trim();
            }
            if (text === "") {
              debugLog('pollAndUpdateLabels: Label key="' + key + '" still empty after update attempt.');
              allUpdated = false;
            } else {
              debugLog('pollAndUpdateLabels: Label key="' + key + '" updated to:', text);
            }
          }
        } else {
          debugLog('pollAndUpdateLabels: Label key="' + key + '" not found.');
          allUpdated = false;
        }
      });
      retries++;
      if (allUpdated) {
        debugLog('pollAndUpdateLabels: All labels updated successfully. Stopping polling.');
        clearInterval(interval);
      } else if (retries >= maxRetries) {
        debugLog('pollAndUpdateLabels: Max retries reached. Stopping polling.');
        clearInterval(interval);
      }
    }, 1000);
  }

  /**
   * preparePush の呼び出し時に、モーダルが表示された後にラベルの更新をポーリングするよう上書きする
   * ※対象の関数が定義されていない場合は、1秒ごとに再試行する
   */
  function waitAndOverridePreparePush() {
    if (window.App && App.Defects && typeof App.Defects.preparePush === 'function') {
      var originalPreparePush = App.Defects.preparePush;
      App.Defects.preparePush = function(...args) {
        debugLog('preparePush: Called with arguments:', args);
        var result = originalPreparePush.apply(this, args);
        debugLog('preparePush: Original function executed. Starting label polling...');
        pollAndUpdateLabels();
        return result;
      };
      debugLog('preparePush を上書きしました。');
    } else {
      console.warn('App.Defects.preparePush が見つかりません。1秒後に再試行します。');
      setTimeout(waitAndOverridePreparePush, 1000);
    }
  }

  if (document.readyState === 'loading') {
    document.addEventListener('DOMContentLoaded', function() {
      run();
      waitAndOverridePreparePush();
    });
  } else {
    run();
    waitAndOverridePreparePush();
  }
})();
